import CodeMirror from 'codemirror';
import Automerge from 'automerge';
import { Change, GetText } from './types';
/**
 * Connect an Automerge document
 *
 * @param notify - a callback that gets called when the doc is updated as the result of an editor change
 * @return ConnectCodeMirror - a function for connecting an Automerge.Text object in the document to a CodeMirror instance
 */
export default class AutomergeCodeMirror<D> {
    private readonly change;
    private readonly mutex;
    private readonly codeMirrorByTextId;
    constructor(change: Change<D>);
    getCodeMirror(textObjectId: Automerge.UUID): CodeMirror.Editor | undefined;
    /**
     * Connects a CodeMirror instance to an Automerge.Text object.
     *
     * @param doc the doc with the text
     * @param codeMirror the editor instance
     * @param getText a function that looks up the Automerge.Text object to connect the CodeMirror editor to
     */
    connectCodeMirror(doc: D, codeMirror: CodeMirror.Editor, getText: GetText<D>): () => void;
    updateCodeMirrors(oldDoc: D, newDoc: D): D;
}
