"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var assert_1 = __importDefault(require("assert"));
require("./codeMirrorEnv");
var automerge_1 = __importDefault(require("automerge"));
var codemirror_1 = __importDefault(require("codemirror"));
var AutomergeCodeMirror_1 = __importDefault(require("../src/AutomergeCodeMirror"));
describe('AutomerCodeMirror', function () {
    var doc;
    var change = function (changeFn) { return (doc = automerge_1.default.change(doc, changeFn)); };
    var getText = function (testDoc) { return testDoc.text; };
    var host;
    var codeMirror;
    beforeEach(function () {
        doc = automerge_1.default.from({ text: new automerge_1.default.Text() });
        host = document.body.appendChild(document.createElement('div'));
        codeMirror = codemirror_1.default(host);
    });
    afterEach(function () {
        host.remove();
    });
    describe('Automerge => CodeMirror', function () {
        it('handles 2 consecutive Automerge changes', function () {
            var automergeCodeMirror = new AutomergeCodeMirror_1.default(change);
            var disconnectCodeMirror = automergeCodeMirror.connectCodeMirror(doc, codeMirror, getText);
            doc = automergeCodeMirror.updateCodeMirrors(doc, automerge_1.default.change(doc, function (proxy) { return proxy.text.insertAt(0, 'hello'); }));
            assert_1.default.strictEqual(codeMirror.getValue(), 'hello');
            doc = automergeCodeMirror.updateCodeMirrors(doc, automerge_1.default.change(doc, function (proxy) { return proxy.text.insertAt(0, 'world'); }));
            assert_1.default.strictEqual(codeMirror.getValue(), 'worldhello');
            disconnectCodeMirror();
        });
        it('ignores Automerge changes after disconnection', function () {
            var automergeCodeMirror = new AutomergeCodeMirror_1.default(change);
            var disconnectCodeMirror = automergeCodeMirror.connectCodeMirror(doc, codeMirror, getText);
            doc = automergeCodeMirror.updateCodeMirrors(doc, automerge_1.default.change(doc, function (proxy) { return proxy.text.insertAt(0, 'hello'); }));
            assert_1.default.strictEqual(codeMirror.getValue(), 'hello');
            disconnectCodeMirror();
            doc = automergeCodeMirror.updateCodeMirrors(doc, automerge_1.default.change(doc, function (proxy) { return proxy.text.insertAt(0, 'world'); }));
            assert_1.default.strictEqual(codeMirror.getValue(), 'hello');
        });
        it('handles document update before CodeMirror is connected', function () {
            var automergeCodeMirror = new AutomergeCodeMirror_1.default(change);
            doc = automergeCodeMirror.updateCodeMirrors(doc, automerge_1.default.change(doc, function (proxy) {
                proxy.text.insertAt(0, 'World');
            }));
            automergeCodeMirror.connectCodeMirror(doc, codeMirror, getText);
            assert_1.default.strictEqual(codeMirror.getValue(), 'World');
            doc = automergeCodeMirror.updateCodeMirrors(doc, automerge_1.default.change(doc, function (proxy) {
                proxy.text.insertAt(0, 'Hello ');
            }));
            assert_1.default.strictEqual(codeMirror.getValue(), 'Hello World');
        });
    });
    describe('CodeMirror => Automerge', function () {
        it('handles 2 consecutive CodeMirror changes', function () {
            var automergeCodeMirror = new AutomergeCodeMirror_1.default(change);
            automergeCodeMirror.connectCodeMirror(doc, codeMirror, getText);
            codeMirror.replaceRange('hello', codeMirror.posFromIndex(0));
            assert_1.default.strictEqual(codeMirror.getValue(), 'hello');
            assert_1.default.strictEqual(doc.text.toString(), 'hello');
            codeMirror.replaceRange('world', codeMirror.posFromIndex(0));
            assert_1.default.strictEqual(codeMirror.getValue(), 'worldhello');
            assert_1.default.strictEqual(doc.text.toString(), 'worldhello');
        });
    });
});
//# sourceMappingURL=AutomergeCodeMirrorTest.js.map